//
// ags_template.cpp : Example AGS plugin file
// See the online API reference for details of how this works.
// Copyright (c) 2002 Chris Jones
//
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define THIS_IS_THE_PLUGIN
#include "agsplugin.h"

// DllMain - standard Windows DLL entry point.
// The AGS editor will cause this to get called when the editor first
// starts up, and when it shuts down at the end.
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved) {

  switch (ul_reason_for_call)	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
  }
  return TRUE;
}

// ***** DESIGN TIME CALLS *******

IAGSEditor *editor;
const char *ourScriptHeader =
  "import int AddNumbers (int, int);\r\n"
  "import int SubtractNumbers (int, int);\r\n";

LPCSTR AGS_GetPluginName(void) {
  // Return the plugin description
	return "Template trivial maths plugin";
}

int  AGS_EditorStartup (IAGSEditor *lpEditor) {
  // User has checked the plugin to use it in their game

  // If it's an earlier version than what we need, abort.
  if (lpEditor->version < 1)
    return -1;

  editor = lpEditor;
  editor->RegisterScriptHeader (ourScriptHeader);

  // Return 0 to indicate success
  return 0;
}

void AGS_EditorShutdown () {
  // User has un-checked the plugin from their game
  editor->UnregisterScriptHeader (ourScriptHeader);
}

void AGS_EditorProperties (HWND parent) {
  // User has chosen to view the Properties of the plugin
  // We could load up an options dialog or something here instead
  MessageBox (parent, "Template plugin, copyright (c) 2002 Chris Jones", "About", MB_OK | MB_ICONINFORMATION);
}

int AGS_EditorSaveGame (char *buffer, int bufsize) {
  // We don't want to save any persistent data
  return 0;
}

void AGS_EditorLoadGame (char *buffer, int bufsize) {
  // Nothing to load for this dummy plugin
}

// ******* END DESIGN TIME  *******


// ****** RUN TIME ********

IAGSEngine *engine;

int AddNumbers (int a, int b) {
  return a + b;
}
int SubtractNumbers (int a, int b) {
  return a - b;
}

void AGS_EngineStartup (IAGSEngine *lpEngine) {
  engine = lpEngine;

  // Make sure it's got the version with the features we need
  if (engine->version < 3) {
    engine->AbortGame ("Engine interface is too old, need newer version of AGS.");
  }

  engine->RegisterScriptFunction ("AddNumbers", AddNumbers);
  engine->RegisterScriptFunction ("SubtractNumbers", SubtractNumbers);
  engine->RequestEventHook (AGSE_POSTSCREENDRAW);
}

void AGS_EngineShutdown() {
  // no work to do here - but if we had created any dynamic sprites,
  // we should delete them here
}

int AGS_EngineOnEvent (int event, int data) {
  if (event == AGSE_POSTSCREENDRAW) {
    // Called just before the real screen is updated
    // This is quite advanced stuff, but it demonstrates how
    // you can basically do whatever you like to the screen
    // with a plugin
    long screenWidth, colDepth, a;
    // Get a reference to the screen we'll draw onto
    BITMAP *virtsc = engine->GetVirtualScreen();
    // Get its surface, in both 8-bit and 16-bit flavours
    unsigned char **charbuffer = engine->GetRawBitmapSurface (virtsc);
    unsigned short **shortbuffer = (unsigned short**)charbuffer;

    // Find out the screen width and colour depth
    engine->GetScreenDimensions (&screenWidth, NULL, &colDepth);

    // Draw a bright blue line across the screen near the top
    for (a = 0; a < screenWidth; a++) {
      if (colDepth <= 8)
        charbuffer[20][a] = 10;
      else if (colDepth <= 16)
        shortbuffer[20][a] = 32155;
      else  // I've been lazy, and haven't implemented 24/32-bit
        engine->AbortGame("This plugin does not support 32-bit graphics");
    }

    // Release the screen so that the engine can continue
    engine->ReleaseBitmapSurface (virtsc);
  }
  return 0;
}

// *** END RUN TIME ****
